#! /usr/bin/env python3

bdf = 'wenquanyi_cjk_basic_9pt.bdf'
cpp = '../../src/bitmapfont_wqy.h'
##cpp = 'bitmapfont_wqy.h'

# FONTBOUNDINGBOX 12 14 0 -3
control_point = 14 - 3
h = 12
w = 12
w_half = 12 // 2

chars = 0
glyphs = []

def getKey(item):
    return item[0]

if __name__ == '__main__':
    with open(bdf) as f:
        new_glyph = False
        glyph_line_count = 0
        glyph = []
        xoff = 0 # 左移位数

        for line in f:
            #print(line)
            if line.find('CHARS ') == 0: #CHARS 30503
                s = line.split()
                #print('CHARS: %s' % s)
                chars = int(s[1])
                #print(chars)

            elif line.find('STARTCHAR ') == 0:
                glyph = [0, 'true', ['0']*w]
                #print(glyph)

            elif line.find('ENCODING ') == 0:
                s = line.split()
                glyph[0] = int(s[1])

            elif line.find('BBX ') == 0:
                bbx = line.split()
                #print(bbx)
                # 计算偏移, 并将该字节左移.
                x_padding = int(bbx[1]) + int(bbx[3])
                if x_padding <= w_half:
                    glyph[1] = 'false'
                    xoff = w_half - x_padding
                else:
                    xoff = w - x_padding
                # 计算高度, 补全收尾行.
                y_padding = int(bbx[4]) + 2 # 底补全
                y_padding_first = h - int(bbx[2]) - y_padding # 顶补全 = 全长(12px) - 字高 - 底补全

                #if xoff < 0 or y_padding_first < 0 or y_padding < 0: print(xoff, y_padding, y_padding_first, glyph[0])
                #if xoff != 1 and y_padding_first != 0 and y_padding != 1: print(xoff, y_padding, y_padding_first, glyph[0])
                #print(glyph[0], ' BBX gen offsite :', xoff, y_padding_first, y_padding)

            elif line.find('BITMAP') == 0:
                new_glyph = True
                glyph_line_count = 0
                if y_padding_first > 0:
                    glyph_line_count = glyph_line_count + y_padding_first
                else:
                    pass # < 0, need skip some lines.

            elif line.find('ENDCHAR') == 0:
                new_glyph = False
                glyph_line_count = 0
                #print('Final Glyph is: %s.' % glyph)
                glyphs.append(glyph)

            elif new_glyph:
                # pack line
                line_final = ''
                line_s = line.strip()
                #print(line_s)
                i = int('0x%s' % line_s, 16) >> 4
                i_str = '{0:0>12b}'.format(i)
                i_str_new = i_str[11] + i_str[10] + i_str[9] + i_str[8] + i_str[7] + i_str[6] + i_str[5] + i_str[4] + i_str[3] + i_str[2] + i_str[1] + i_str[0]
                i_int_new = int('0b%s' % i_str_new, 2)
                #print('{0:0>12b}'.format(i_int_new))
                #if xoff > 0: pass
                #if xoff < 0: pass
                #print('glyph_line_count', glyph_line_count)
                glyph[2][glyph_line_count] = str(i_int_new)
                glyph_line_count = glyph_line_count + 1

    glyphs.sort(key=getKey)

    fw = open(cpp, 'w')
    fw.write('''/* !!!! GENERATED FILE - DO NOT EDIT !!!!
 * --------------------------------------
 */
#ifndef EP_BITMAPFONT_WQY_H
#define EP_BITMAPFONT_WQY_H

#include <array>
#include "bitmapfont_glyph.h"

constexpr const std::array<BitmapFontGlyph,%s> BITMAPFONT_WQY = {{
''' % len(glyphs))
    for x in glyphs:
        #print(x[0])
        #print(', '.join(x[2]))
        s = '\t{ %s, %s, { %s } },\n' % (x[0], x[1], ', '.join(x[2]))
        #print(s)
        fw.write(s)

    fw.write('''}};

#endif
''')
    fw.close()




#!/bin/env python3

# Script to generate all combinations for the CMakePresets.json
# Edit the CMakePresets.json.template and run this script
# License for the script and all json files including generated: Unlicense
# (c) Ghabry

from copy import deepcopy
import json
import os

script_dir = os.path.dirname(os.path.abspath(__file__))
repo_dir = f"{script_dir}/../.."

with open(f"{script_dir}/CMakePresets.json.template", "r") as f:
	j = json.load(f)

conf_presets = j["configurePresets"]
conf_presets_out = []

def append_name(name):
	if len(item["name"]) > 0:
		item["name"] += "-"
	item["name"] += name

platform_display = dict(
	sdl1="SDL1",
	sdl2="SDL2",
	sdl3="SDL3",
	libretro="libretro core"
)

# This creates the following configurePresets from the one in the template:
# - As specified in the template
# - For every entry in easyrpg_platforms (N)
# For all of them the build types Debug, RelWithDebInfo and Release are generated.
# Making this (N+1) * 3 entries per preset.

# The resulting "triplet" is always:
# {name_from_template}-{libretro}-{build_type}
# Omit libretro to disable it

# The build dirs are always:
# build/{name_from_template}-{libretro}-{build_type}

for base_item in conf_presets:
	if base_item.get("hidden"):
		conf_presets_out.append(base_item)
		continue

	# Create "base class" the build types inherit from
	item = deepcopy(base_item)
	append_name("parent")
	del item["displayName"]
	item["hidden"] = True

	if item.get("inherits") is None:
		item["inherits"] = "base-user"

	ep_platforms = ["default"]
	if "easyrpg_platforms" in item:
		ep_platforms += item["easyrpg_platforms"]
		del item["easyrpg_platforms"]

	parent_item = deepcopy(item)
	conf_presets_out.append(parent_item)

	for platform in ep_platforms:
		# Ugly: Generates a huge amount of configurePresets
		# Cannot be improved until limitations in buildPresets are resolved
		# (see comment below)
		for build_type in ["Debug", "RelWithDebInfo", "Release"]:
			item = dict(name=base_item["name"], displayName=base_item["displayName"])
			name = item["name"]

			item["inherits"] = [parent_item["name"]]

			if platform != "default":
				append_name(platform)
				item["inherits"].insert(0, f"build-{platform}")
				item["displayName"] += f" ({platform_display[platform]}, {build_type})"
			else:
				item["displayName"] += f" ({build_type})"

			item["inherits"] += [f"type-{build_type.lower()}"]

			append_name(build_type.lower())

			conf_presets_out.append(item)

j["configurePresets"] = conf_presets_out

bp = j["buildPresets"]
for item in conf_presets_out:
	if item.get("hidden"):
		continue

	# Here it would make sense to provide the build types, however this will
	# only work for Multi Config generators.
	# Instead the build types are generated as part of the configurePreset.
	# Code kept for reference in case CMake improves this in a later version.

	#for build_type in ["Debug", "RelWithDebInfo", "Release", "MinSizeRel"]:
	#	name = item["name"] + f"-{build_type.lower()}"
	#	conf_preset = item["name"]
	#	bp.append(dict(
	#		name=name,
	#		displayName=build_type,
	#		configurePreset=conf_preset,
	#		configuration=build_type))

	bp.append(dict(name=item["name"], configurePreset=item["name"]))

# Add note that the file is auto-generated to the beginning
vendor = dict(vendor=dict(
	README=dict(
		line1="!!! GENERATED FILE. DO NOT EDIT !!!",
		line2="Modify builds/cmake/CMakePresets.json.template instead",
		line3="and run gen-cmake-presets.py to regenerate this file."
	)
))

j = {**vendor, **j}

with open(f"{repo_dir}/CMakePresets.json", "w") as f:
	json.dump(j, f, indent='\t')
	f.write("\n")



#!/usr/bin/env python3
# Usage example:
# ./rtp.py 0 < RTP2k.csv > rtp.cpp
# ./rtp.py 1 < RTP2k3.csv >> rtp.cpp

from sys import stdin, argv

if len(argv) < 2:
	raise ValueError("Specify 0 (2k) or 1 (2k3)")

rtp_table = "3" if argv[1] == "1" else ""

firstline = stdin.readline()
elems = len(firstline.split(","))
first = ""
lines = []
lookup = {}

for i, line in enumerate(stdin):
	l = line.lower().strip().split(",")
	lst = [""]*elems
	if len(l[0]) > 0:
		first = l[0]
		lookup[first] = i
	lst[0] = first
	lst[1:] = l[1:]
	while len(lst) < elems:
		lst.append("")
	lines.append(lst)

print("const char* const rtp_table_2k%s[][%s] = {" % (rtp_table, str(elems)))
for l in lines:
	print("\t{", end="")
	for i in range(elems):
		ll = l[i]
		if len(ll) == 0:
			if i == (elems - 1):
				print("nullptr", end="")
			else:
				print("nullptr, ", end="")
		elif i == (elems - 1):
			s = '"' + ll + '"'
			print(s, end="")
		else:
			s = '"' + ll + '"'
			print(s + ', ', end="")

	print("},")

print("	{", end="")
for i in range(elems):
	if i == elems - 1:
		print("nullptr", end="")
	else:
		print("nullptr, ", end="")
print("}")
print("};")

print("");
print("const char* const rtp_table_2k%s_categories[%s] = {" % (rtp_table, len(lookup.items()) + 1))

for k in lookup.keys():
	print('\t"%s",' % k)

print("\tnullptr")
print("};")

print("")
print("const int rtp_table_2k%s_categories_idx[%s] = {" % (rtp_table, len(lookup.items()) + 1))

for v in lookup.values():
	print('\t%s,' % v)

print("\t%s" % len(lines))
print("};")
print("")
